
require("cross-fetch/polyfill");
require("isomorphic-form-data");

const express = require("express");
const session = require("express-session");
const FileStore = require("session-file-store")(session);
const app = express();
const { ArcGISIdentityManager } = require("@esri/arcgis-rest-request");

const { CLIENT_ID, SESSION_SECRET, ENCRYPTION_KEY, REDIRECT_URI } = require("./config.json");

const credentials = {
  clientId: CLIENT_ID,
  redirectUri: REDIRECT_URI
};

app.use(
  session({
    name: "ArcGIS REST JS server authentication tutorial",
    secret: SESSION_SECRET,
    resave: false,
    saveUninitialized: false,
    cookie: {
      maxAge: 2592000000 // 30 days in milliseconds
    },

    store: new FileStore({
      ttl: 2592000000 / 1000, // 30 days in seconds
      retries: 1,
      secret: ENCRYPTION_KEY,

      // define how to encode our session to text for storing in the file. We can use the `serialize()` method on the session for this.
      encoder: (sessionObj) => {
        console.log({ sessionObj });
        sessionObj.arcgis = sessionObj.arcgis.serialize();
        return JSON.stringify(sessionObj);
      },

      // define how to turn the text from the session data back into an object. We can use the `ArcGISIdentityManager.deserialize()` method for this.
      decoder: (sessionContents) => {
        console.log({ sessionContents });
        const sessionObj = JSON.parse(sessionContents);
        if (sessionObj.arcgis) {
          sessionObj.arcgis = ArcGISIdentityManager.deserialize(sessionObj.arcgis);
        }

        return sessionObj;
      }

    })

  })
);

app.get("/sign-in", (req, res) => {
  ArcGISIdentityManager.authorize(credentials, res);
});

app.get("/sign-out", (req, res) => {
  // invalidate the token
  ArcGISIdentityManager.destroy(req.session.arcgis).then(() => {
    // delete the cookie and stored session
    req.session.destroy();
    res.redirect("/");
  });
});

app.get("/authenticate", async (req, res) => {
  req.session.arcgis = await ArcGISIdentityManager.exchangeAuthorizationCode(
    {
      clientId: CLIENT_ID,
      redirectUri: REDIRECT_URI
    },
    req.query.code //The code from the redirect: exchange code for a token in instaniated user session.
  );

  res.redirect("/");

});

app.get("/", (req, res) => {
  //Redirect to homepage.
  if (req.session.arcgis) {
    res.send(`
    <h1>Hi ${req.session.arcgis.username}<h1>
    <pre><code>${JSON.stringify(req.session.arcgis, null, 2)}</code></pre>
    <a href="/sign-out">Sign Out<a>
  `);
  } else {
    res.send(`<a href="/sign-in">Sign In<a>`);
  }
});

app.listen(3000, () => {
  console.log(`Visit http://localhost:3000/ to get started!`);
});


